using UnityEngine;
using UnityEngine.PostProcessing;
using System;
using System.Collections.Generic;
using UnityEngine.UI;
using MVR.FileManagementSecure;

namespace MacGruber
{
	namespace PostMagic 
	{		
		public class UserLUT : MVRScript
		{
			private Manager manager;
			private UserLutModel model;
			private UserLutModel.Settings settings;
			
			private JSONStorableBool active;
			private JSONStorableFloat contribution;
			private JSONStorableBool bilinearFiltering;			
			private JSONStorableUrl textureURL;
			private JSONStorableStringChooser textureChooser;
			
			private bool textureChooserIgnoreCallback = false;
			
			private Texture2D texture;
			
			public override void Init()
			{
				manager = Utils.FindWithinSamePlugin<Manager>(this);
				model = manager.profile.userLut;
				
				Utils.SetupInfoText(this, 
					"<b>Contribution:</b> Blending factor.\n\n" + 					
					"<b>Bilinear Filtering:</b> Switch between Point and Bilinear filter modes.\n\n" +
					"<b>UserLUT:</b> Custom color lookup texture in strip format, e.g. 256x16 or 1024x32.\n\n" +
					"UserLUT is a simple method of color grading where pixels on screen are replaced by new values from an LUT (or look-up texture) supplied by the user. To create an LUT import one of the neutral LUTs into an image editing tool such as Photoshop with a screenshot of your scene. Apply color corrections in a non destructive manner on top of these two images until you are happy with the result. Note that only pixel-local effects are supported by LUTs, meaning no blur and other effects that depends on the value of neighboring pixels. Now export the LUT with these color changes applied back into VaM to be used in the UserLUT effect.\n\n" +
					"The dropdown menu shows the image files in the current folder for quick selection.\n\n" +
					"<b>Next/Previous UserLUT:</b> Cycle through the current folder. These buttons are also exposed as Trigger, so you can setup for example buttons in your scene or keybindings (via plugin).",
					1140.0f, true
				);
				
				active = Utils.SetupToggle(this, "UserLUT Enabled", false, false);
				active.setCallbackFunction  += (bool v) => { model.enabled = enabledJSON.val = v; };
				
				contribution = Utils.SetupSliderFloat(this, "Contribution", 1.0f, 0.0f, 1.0f, false);
				contribution.setCallbackFunction  += (float v) => { settings.contribution = v; model.settings = settings; };
				
				bilinearFiltering = Utils.SetupToggle(this, "Bilinear Filtering", true, false);
				bilinearFiltering.setCallbackFunction  += (bool v) => { textureURL.setCallbackFunction(textureURL.val); };
				
				CreateSpacer(false).height = 30.0f;		
										
				string defaultTex = Utils.GetPackagePath(this) + "Custom/Assets/MacGruber/PostMagic/LUT32/NeutralLUT32.png";
				TextureSettings textureSettings = new TextureSettings() { createMipMaps=false, linearColor=true, wrapMode=TextureWrapMode.Clamp, anisoLevel=0 };
				textureURL = Utils.SetupTexture2DChooser(this, "UserLUT", defaultTex, false, textureSettings, TextureLoaded);
				
				
				// Creating StringChooser with hidden text label on the left to get more space				
				List<string> choices = new List<string>();
				List<string> display = new List<string>();
				textureChooser = new JSONStorableStringChooser("UserLUT QuickSelect", choices, display, "", "");
				textureChooser.isStorable = textureChooser.isRestorable = false;
				CreateScrollablePopup(textureChooser, false);
				textureChooser.setCallbackFunction  += (string v) => {
					if (!textureChooserIgnoreCallback)
						textureURL.val = v;
				};
				Transform popup = textureChooser.popup.transform;
				popup.Find("Slider").GetComponent<RectTransform>().offsetMin = new Vector2(0.0f, -35.0f);
				popup.Find("Button").GetComponent<RectTransform>().offsetMin = new Vector2(0.0f, 5.0f);
				popup.Find("PopupPanel").GetComponent<RectTransform>().offsetMin = new Vector2(0.0f, -345.0f);
				popup.Find("Text").gameObject.SetActive(false);
				
				
				

				Utils.SetupButton(this, "Next UserLUT", OnNextLUT, false);
				Utils.SetupButton(this, "Previous UserLUT", OnPreviousLUT, false);
				Utils.SetupAction(this, "NextUserLUT", OnNextLUT);
				Utils.SetupAction(this, "PreviousUserLUT", OnPreviousLUT);
				
				
				
				settings = UserLutModel.Settings.defaultSettings;
				settings.contribution = contribution.val;
				model.settings = settings;
				
				active.setCallbackFunction(active.val);
				enabledJSON.setCallbackFunction += (bool v) => { active.val = v; };
			}
			
			private void OnDestroy()
			{
				if (texture != null)
					Destroy(texture);
				texture = null;
			}
						
			private void TextureLoaded(Texture2D tex)
			{		
				if (texture != null)
					Destroy(texture);
				settings.lut = texture = tex;
				if (tex != null)
					tex.filterMode = bilinearFiltering.val ? FilterMode.Bilinear : FilterMode.Point;
				model.settings = settings;
				
				
				// update quick selection textureChooser
				string folder = FileManagerSecure.GetDirectoryName(textureURL.val);
				string[] files = FileManagerSecure.GetFiles(folder);
				List<string> choices = textureChooser.choices;
				List<string> display = textureChooser.displayChoices;
				choices.Clear();
				display.Clear();
				for (int i=0; i<files.Length; ++i)
				{
					int idx = Mathf.Max(files[i].Length - 5, 0);
					string f = files[i].Substring(idx).ToLower();
					if (!f.EndsWith(".jpg") && !f.EndsWith(".png") && !f.EndsWith(".tif") && !f.EndsWith(".tiff"))
						continue;
					
					string filepath = FileManagerSecure.NormalizePath(files[i]);	
					choices.Add(filepath);
				}
				choices.Sort();
				for (int i=0; i<choices.Count; ++i)
				{
					string filename = FileManagerSecure.GetFileName(choices[i]);				
					display.Add(filename);
				}				
				textureChooser.choices = null;
				textureChooser.displayChoices = display;
				textureChooser.choices = choices;
				
				textureChooserIgnoreCallback = true;
				textureChooser.valNoCallback = ""; // force slider to update
				textureChooser.val = textureURL.val;
				textureChooserIgnoreCallback = false;
			}
			
			private void OnNextLUT()
			{
				ChangeLUT(1);
			}
			
			private void OnPreviousLUT()
			{
				ChangeLUT(-1);
			}
			
			private void ChangeLUT(int offset)
			{
				List<string> choices = textureChooser.choices;
				if (choices.Count == 0)
					return;
				
				int index = choices.IndexOf(textureChooser.val);
				if (index < 0)
					index = 0;
				else
					index += offset;
				
				if (index < 0)
					index = choices.Count - 1;
				else if (index >= choices.Count)
					index = 0;
					
				textureChooser.val = choices[index];
			}
		}
	}
}